//  BASELIB.PRG
//  Jonathan Cohen
//  CIS 75360,2005

//     This group of Q&D functions was created
//     out of necessity, but may be of some help
//     to those of you who work with hex or binary
//     data.  There seems to be very little out there
//     on using anything other than base 10 with FoxPro
//     without going through the labor of learning the LCK.

// Conversion to Clipper Code by Frank W. Moore
// CIS 74351, 1527

// Adapted these functions to native clipper code, and added 
// the ability to handle negative numbers.  The conversion is ONLY
// for INTEGER values

// Examples:

// SET TALK off

// ? DecToHex(991231235959)
// ? HexToDec('-AB04')

// ? DecToBin(-100)
// ? BinToDec('0111')


FUNCTION DecToBin(nInnum)
// Converts decimal integers to BIN.
//  Input:    NUMERIC
// Output:    CHAR

     LOCAL cOutstr := '', lNeg := .F.
     
     IF nInnum < 0
          nInnum := nInnum * -1
          lNeg := .T.
     ENDIF

     DO WHILE nInNum > 0
               cOutStr := STR(MOD(nInNum, 2), 1, 0) + cOutstr
               nInNum := INT(nInNum/2)
     ENDDO

     // Add '-' if negative
     IF lNeg
          cOutstr := '-' + cOutstr
     ENDIF

RETURN(cOutStr)

FUNCTION BinToDec(cInstr)
// Converts binary numbers (represented by Char) to decimal integer.
//  Input:    CHAR
// Output:    NUMERIC

     LOCAL nJnsum := 0, nJnlen := LEN(ALLTRIM(cInstr)), nCtr := 0, ;
          lNeg := .F.

     // Check for negative Number
     IF LEFT(cInStr, 1) == '-'
          cInStr := RIGHT(cInStr, (nJnLen - 1))
          nJnLen := LEN(cInStr)
          lNeg := .T.
     ENDIF
     
     FOR nCtr := 0 TO nJnlen
          IF SUBSTR(cInstr, nJnlen - nCtr, 1) == '1'
               nJnsum := nJnsum + (2 ^ nCtr)
          ENDIF
     NEXT
     
     // Make negative if required
     IF lNeg
          nJnsum := nJnSum * -1
     ENDIF
     
RETURN(INT(nJnsum))

FUNCTION DecToHex(nInnum)
// Converts decimal integers to hex (Char).
//  Input:    NUMERIC
// Output:    CHAR

     LOCAL cOutstr := '', lNeg := .F.
     
     IF nInnum < 0
          nInnum := nInnum * -1
          lNeg := .T.
     ENDIF
     
     DO WHILE nInnum >  0
               cOutStr := FindHex(MOD(nInnum, 16)) + cOutstr                   
               nInnum := INT(nInnum / 16)
     ENDDO
     
     // Add '-' if negative
     IF lNeg
          cOutstr := '-' + cOutstr
     ENDIF

RETURN(cOutstr)

FUNCTION FindHex(nInVal)
// Lookup table for conversion of alpha hex chars.
//  Input:    NUMERIC
// Output:    CHAR

     LOCAL cjOutStr := ''

     DO CASE
          CASE nInVal == 10
               cjOutStr := 'A'
          CASE nInVal == 11
               cjOutStr := 'B'
          CASE nInVal == 12
               cjOutStr := 'C'
          CASE nInVal == 13
               cjOutStr := 'D'
          CASE nInVal == 14
               cjOutStr := 'E'
          CASE nInVal == 15
               cjOutStr := 'F'
          OTHERWISE
               cjOutStr := STR(nInVal, 1, 0)
     ENDCASE
RETURN(cjOutStr)

FUNCTION HexToDec(cInStr)
//  This function converts Char to decimal. 
//  Input:    CHAR
// Output:    NUMERIC

     LOCAL njLen := LEN(cInStr), nSum := 0, nrPtr:= 0, nCtr := 0, ;
          cPtr := '', lNeg := .F.
     
     cInStr := ALLTRIM(cInStr) 
     
     // Check for negative Number
     IF LEFT(cInStr, 1) == '-'
          cInStr := RIGHT(cInStr, (njLen - 1))
          njLen := LEN(cInStr)
          lNeg := .T.
     ENDIF
     
     FOR nCtr := 1 TO njLen
          cPtr := UPPER(SUBSTR(cInStr, njLen - nrPtr, 1))
          DO CASE
          CASE cPtr == 'A' 
               cPtr := '10'           
          CASE cPtr == 'B' 
               cPtr := '11'           
          CASE cPtr == 'C' 
               cPtr := '12'           
          CASE cPtr == 'D'
               cPtr := '13'           
          CASE cPtr == 'E'
               cPtr := '14'           
          CASE cPtr == 'F'
               cPtr := '15'           
          ENDCASE
          nSum := nSum + (VAL(cPtr) * 16 ^ (nCtr - 1)) 
          nrPtr := nrPtr + 1
     NEXT
     
     // Set negative if necessary
     IF lNeg
          nSum := nSum * -1
     ENDIF

RETURN(INT(nSum))
